#関連パッケージのimport
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from pandas import Series, DataFrame
from numpy.random import normal
import os
import sys
import plotly
import plotly.graph_objects as go
import plotly.express as px
import configparser
import errno


def trim_string(string):
    return string.strip()

CONFIG_FILE_NAME = 'Sturtevant.ini'
if not os.path.exists(CONFIG_FILE_NAME):
    raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), CONFIG_FILE_NAME)


# Confファイルの読み込み
print("Confファイルの読み込み Start")
config_ini = configparser.ConfigParser()
config_ini.read(CONFIG_FILE_NAME, encoding='utf-8')
ExcelFileDirectory = config_ini['ExcelFile']['ExcelFileDirectory']
ltxMetricsFileName = config_ini['ExcelFile']['ltxMetricsFileName']


sheetName_System = config_ini['ExcelSheetName']['sheetName_System']
sheetName_Architecture = config_ini['ExcelSheetName']['sheetName_Architecture']
sheetName_Object = config_ini['ExcelSheetName']['sheetName_Object']


sheetName_System_skiprows = int(config_ini['ExcelSheetFormat']['sheetName_System_skiprows'])
sheetName_Architecture_skiprows = int(config_ini['ExcelSheetFormat']['sheetName_Architecture_skiprows'])
sheetName_Object_skiprows = int(config_ini['ExcelSheetFormat']['sheetName_Object_skiprows'])

columnName_SubSystem = config_ini['ColumnName']['columnName_SubSystem']
columnName_HeaderFile = config_ini['ColumnName']['columnName_HeaderFile']
columnName_SourceFile = config_ini['ColumnName']['columnName_SourceFile']
columnName_ElementCount = config_ini['ColumnName']['columnName_ElementCount']
columnName_EikyodoHeikin = config_ini['ColumnName']['columnName_EikyodoHeikin']
columnName_HeikinRuisekiIzondo = config_ini['ColumnName']['columnName_HeikinRuisekiIzondo']
columnName_CyclomaticComplexity = config_ini['ColumnName']['columnName_CyclomaticComplexity']

devide_value_of_maxVFI = int(config_ini['Threshold']['devide_value_of_maxVFI'])
devide_value_of_maxVFO = int(config_ini['Threshold']['devide_value_of_maxVFO'])
cyclomaticComplexity_Medium_min     = int(config_ini['Threshold']['cyclomaticComplexity_Medium_min'])
cyclomaticComplexity_High_min       = int(config_ini['Threshold']['cyclomaticComplexity_High_min'])
cyclomaticComplexity_Untestable_min = int(config_ini['Threshold']['cyclomaticComplexity_Untestable_min'])


print("Confファイルの読み込み End")

#Lattixメトリクスレポート(Excel形式)の読み込み
print("Lattixメトリクスレポート(Excel形式)の読み込み Start")

dirPath = ExcelFileDirectory
filePath = os.path.join(dirPath, ltxMetricsFileName)
ltx_org_xlsxFile = pd.ExcelFile(filePath)

df_ltx_org_System = ltx_org_xlsxFile.parse(sheetName_System,skiprows = sheetName_System_skiprows)
df_ltx_org_Architecture = ltx_org_xlsxFile.parse(sheetName_Architecture,skiprows = sheetName_Architecture_skiprows)
df_ltx_org_Object = ltx_org_xlsxFile.parse(sheetName_Object,skiprows = sheetName_Object_skiprows)

print("Lattixメトリクスレポート(Excel形式)の読み込み End")

#メトリクスのマージ
df_ltx_org=pd.merge(df_ltx_org_System,df_ltx_org_Architecture, on=columnName_SubSystem)

# 使用するメトリクスの指定
default_culumn = [columnName_SubSystem,columnName_HeaderFile,columnName_SourceFile,columnName_ElementCount]
metrics = [columnName_EikyodoHeikin, columnName_HeikinRuisekiIzondo, columnName_CyclomaticComplexity]
culumn_use = default_culumn + metrics
df_ltx=df_ltx_org.loc[:,culumn_use]

# ファイル要素のみ抽出（サブシステムに対するメトリクスデータを除外）
df_ltx_file=df_ltx[df_ltx[columnName_ElementCount]==1.0]
df_ltx_file[columnName_SubSystem]=df_ltx_file[columnName_SubSystem].apply(trim_string)
df_ltx_file=df_ltx_file[~df_ltx_file[columnName_SubSystem].str.contains(r"\.\*$")]

# VFI,VFOの計算(VFI = 影響度平均+1, VFO = 平均累積依存度)
df_ltx_file.loc[:,"VFI"]=pd.DataFrame(df_ltx_file[columnName_EikyodoHeikin].copy(deep=True)+1)
df_ltx_file.loc[:,"VFO"]=pd.DataFrame(df_ltx_file[columnName_HeikinRuisekiIzondo].copy(deep=True))
df_ltx_file.loc[:,"SQRT(VFI*VFO)"]=pd.DataFrame(np.sqrt(df_ltx_file["VFI"].copy(deep=True)*df_ltx_file["VFO"].copy(deep=True)))



# VFI,VFOの閾値を計算（ここでは最大値のN分の1と定義）
threshold_VFI = df_ltx_file["VFI"].max()/devide_value_of_maxVFI
threshold_VFO = df_ltx_file["VFO"].max()/devide_value_of_maxVFO

groupname = {0:"0_periferal",1:"1_utility",2:"2_control",3:"3_core"}  
cyc_groupname = {0:"0_nothing", 1:"1_low", 2:"2_medium", 3:"3_high", 4:"4_untestable"}

def judge_vfio(df_input):
    judgeVFI = df_input["VFI"].apply(lambda x: 1 if (x>threshold_VFI) else 0)
    judgeVFO = df_input["VFO"].apply(lambda x: 2 if (x>threshold_VFO) else 0)
    return judgeVFI+judgeVFO

def judge_cyc(df_input):
    low_min=1
    medium_min = cyclomaticComplexity_Medium_min
    high_min = cyclomaticComplexity_High_min
    untestable_min = cyclomaticComplexity_Untestable_min
    low = df_input[columnName_CyclomaticComplexity].apply(lambda x: 1 if (x>=low_min) else 0)
    medium = df_input[columnName_CyclomaticComplexity].apply(lambda x: 1 if (x>=medium_min) else 0)
    high = df_input[columnName_CyclomaticComplexity].apply(lambda x: 1 if (x>=high_min) else 0)
    untestable = df_input[columnName_CyclomaticComplexity].apply(lambda x: 1 if (x>=untestable_min) else 0)
    return low+medium+high+untestable

def get_groupName(df_input):
    return df_input["group"].apply(lambda x: groupname[x])

def get_cycgroupName(df_input):
    return df_input["cyc_group"].apply(lambda x: cyc_groupname[x])
                                 
df_ltx_file.loc[:,"group"]=judge_vfio(df_ltx_file)
df_ltx_file.loc[:,"groupName"]=get_groupName(df_ltx_file)
df_ltx_file.loc[:,"cyc_group"]=judge_cyc(df_ltx_file)
df_ltx_file.loc[:,"cyc_groupName"]=get_cycgroupName(df_ltx_file)

print("閾値:VFO:"+str(threshold_VFO)+" "+"VFI:"+str(threshold_VFI))


# 構造複雑度別に各ファイルをプロットする関数
def plot_xy(x_metrics,y_metrics,df,title=""):
    if title == "":
        title = str(x_metrics)+"-"+str(y_metrics)+"のグラフ(構造複雑度別)"
    x_metrics = x_metrics
    y_metrics = y_metrics

    x = df[x_metrics]
    y = df[y_metrics]
    group = df["group"]

    import plotly.express as px

    symbols =["circle","square","star","x","cross"]
    data = [
        go.Scatter(
            x=x[group==i], y=y[group==i],
            name=groupname[i],
            mode = 'markers',
            marker = dict(color=px.colors.qualitative.Plotly[i],
                size=6,
                symbol='star'
                ),
            text = df[columnName_SubSystem][group==i]
        )
        for i in range(len(groupname))
    ]

    fig = go.Figure(data=data)
    fig.update_xaxes(title_text=x_metrics)
    fig.update_yaxes(title_text=y_metrics, hoverformat=".3f")
    fig.update_layout(title=dict(text='<b>'+title,
                             font=dict(size=26,
                                       color='black'),
                             xref='paper',
                             x=0.5,
                             y=0.9,
                             xanchor='center'
                            ))

    #plotly.offline.iplot(fig)
    return fig
# クロス集計した結果をプロットする関数
def plot_xy_cross(x_metrics,y_metrics,df,title=""):
    if title == "":
        title = str(x_metrics)+"-"+str(y_metrics)+"のグラフ(クロス集計)"
    x_metrics = x_metrics
    y_metrics = y_metrics

    x = df[x_metrics]
    y = df[y_metrics]
    group = df["group"]
    cyc_group = df["cyc_group"]

    symbols =["circle","square","star","x","cross"]
    data = [
        go.Scatter(
            x=x[cyc_group==i][group==j], y=y[cyc_group==i][group==j],
            name=cyc_groupname[i]+" & "+groupname[j],
            mode = 'markers',
            marker = dict(color=px.colors.qualitative.Plotly[i],
                size=4,
                symbol=symbols[j]
                ),
            text = df[columnName_SubSystem][group==j][cyc_group==i]
        )
        for i in range(len(cyc_groupname)) for j in range(len(groupname))
        
    ]

    fig = go.Figure(data=data)
    fig.update_xaxes(title_text=x_metrics)
    fig.update_yaxes(title_text=y_metrics, hoverformat=".3f")
    fig.update_layout(title=dict(text='<b>'+title,
                             font=dict(size=26,
                                       color='black'),
                             xref='paper', # container or paper
                             x=0.5,
                             y=0.9,
                             xanchor='center'
                            ))

    return fig

# VFI-VFOグラフの描画
print("グラフ出力 Start")

fig = plot_xy("VFI","VFO",df_ltx_file,"VFIとVFOの各ファイル分布(構造複雑度別)")
fig.write_html("VFIとVFOの各ファイル分布(構造複雑度別).html")

fig = plot_xy_cross("VFI","VFO",df_ltx_file,"VFIとVFOの各ファイル分布(クロス集計)")
fig.write_html("VFIとVFOの各ファイル分布(クロス集計).html")

#VFI/VFO-Cyclomaticグラフの描画
fig = plot_xy("SQRT(VFI*VFO)","Cyclomatic Complexity",df_ltx_file,"構造複雑度とCyclomatic複雑度の各ファイル分布(構造複雑度別)")
fig.write_html("構造複雑度とCyclomatic複雑度の各ファイル分布(構造複雑度別).html")

fig = plot_xy_cross("SQRT(VFI*VFO)","Cyclomatic Complexity",df_ltx_file,"構造複雑度とCyclomatic複雑度の各ファイル分布(クロス集計)")
fig.write_html("構造複雑度とCyclomatic複雑度の各ファイル分布(クロス集計).html")

print("グラフ出力 End")

#構造複雑度とCyclomatic複雑度のファイル数
#各グループに属するファイルの個数
df_crosstab=pd.crosstab(df_ltx_file["groupName"],df_ltx_file["cyc_groupName"],margins=True)

#各グループに属するファイルの割合（百分率）
df_crosstab_norm=pd.crosstab(df_ltx_file["groupName"],df_ltx_file["cyc_groupName"],normalize=True,margins=True)



# 1カラムあたりの文字数変更
pd.set_option("display.max_colwidth", 80)

#csv出力

print("CSV出力 Start")

df_ltx_file.to_csv('df_Sturtevant.csv',sep=',',encoding='cp932')
#df_ltx_file[(df_ltx_file["groupName"]=="2_control") & (df_ltx_file["cyc_groupName"]=="4_untestable")].to_csv('df_Sturtevant_control_untestable.csv',sep=',',encoding='cp932')

df_crosstab.to_csv('df_Sturtevant_crosstab.csv',sep=',',encoding='cp932')

df_crosstab_norm = df_crosstab_norm.apply(lambda x: x.map("{0:.2%}".format))
df_crosstab_norm.to_csv('df_Sturtevant_crosstab_norm.csv', sep=',', encoding='cp932')

print("CSV出力 End")

